# # spell-checker:words vswr
# import typer
# from enum import StrEnum
# from rich import print
# from rich.panel import Panel
# from rich.table import Table
# import numpy as np
# from scipy.constants import c, epsilon_0 as e0
# from typing import Union


# welcome_horse = r"""

#                                        ._ o o
#                                        \_`-)|_
#                                     ,""       \
#                                   ,"  ## |   ಠ ಠ.
#                                 ," ##   ,-\__    `.
#                               ,"       /     `--._;)
#                             ,"     ## /
#                           ,"   ##    /
# """

# rect_dr_graph = r"""
#                               ----------------------------|
#                              /                           /|
#                             /   Rectangular Resonator   / |
#                            /                           /  |
#                         ^ |---------------------------/   |
#                 --------|--                           |   |----------------
#                /        | |                           |   -              /
#  z            /         | |            ε_r            |  /  ^           /
#              /   height | |                           | /  /           /
#  ^          /           | |                           |/  / depth     /
#  |    y    /            v ----------------------------/  v           /
#  |   ^    /               <--------------------------->             /
#  |  /    /                           width                         /
#  | /    /---------------------------------------------------------/
#  |/
#  |---------> x             infinite ground plane
# """


# class OpMode(StrEnum):
#     """Operation mode:
#     - `TExd11`: TE(x)d11 mode
#     - `TEy1d1`: TE(y)1d1 mode
#     - `TEz11d`: TE(z)11d mode (isolated resonator)
#     """

#     TExd11 = "TExd11"
#     TEy1d1 = "TEy1d1"
#     TEz11d = "TEz11d"


# def print_graph():
#     print(
#         Panel.fit(
#             rect_dr_graph,
#             title="Rectangular Resonator Dimensions",
#             title_align="left",
#             subtitle="""Default length unit: mm""",
#             subtitle_align="left",
#         )
#     )


# def solve_TExd11(
#     f: np.double, epsilon_r: np.double, w_h: np.double, d_h: np.double, k0: np.double
# ):
#     """Solve TE(x)d11 mode.

#     Args:
#     -----
#         - f (np.double): frequency
#         - epsilon_r (np.double): dielectric constant
#         - w_h (np.double): width/height ratio
#         - d_h (np.double): depth/height ratio
#         - k0 (np.double): wave number

#     Returns:
#     --------
#         - _type_: _description_
#     """
#     d = 0
#     for n in np.arange(0, np.Inf, 0.0001):
#         ky = np.pi * d_h / w_h / n
#         kz = np.pi * d_h / n
#         kx = np.sqrt(epsilon_r * k0**2 - ky**2 - kz**2)
#         y = kx * np.tan(kx * n / 2) - np.sqrt((epsilon_r - 1) * k0**2 - kx**2)
#         if y > 0:
#             d = n
#             break
#     b = d / d_h
#     h = b / 2
#     w = b * w_h
#     ky = np.pi / w
#     kz = np.pi / b
#     kx = np.sqrt(epsilon_r * k0**2 - ky**2 - kz**2)
#     we = (
#         e0
#         * epsilon_r
#         * w
#         * 2
#         * h
#         * d
#         / 32
#         * (1 + np.sin(kx * d) / kx / d)
#         * (ky**2 + kz**2)
#     )
#     pm = (
#         -1j
#         * 2
#         * np.pi
#         * f
#         * 1e7
#         * 8
#         * e0
#         * (epsilon_r - 1)
#         / kx
#         / ky
#         / kz
#         * np.sin(kx * d / 2)
#     )
#     k0 = np.sqrt((kx**2 + ky**2 + kz**2) / epsilon_r)
#     p_rad = 10 * k0**4 * np.linalg.norm(pm) ** 2
#     q = 4 * np.pi * f * 1e7 * we / p_rad

#     return w, d, h, q


# app = typer.Typer()


# @app.callback()
# def rect(
#     show_graph: bool = typer.Option(
#         False,
#         "--show-graph",
#         "-g",
#         help="""Show a graph about dimensions of a rectangular resonator""",
#     )
# ):
#     if show_graph:
#         print_graph()


# @app.command(name="design")
# def cli_design(
#     freq: float = typer.Option(
#         24,
#         help="Resonant frequency (GHz)",
#         prompt="""1. Enter the desired resonant frequency (GHz)""",
#     ),
#     mode: OpMode = typer.Option(
#         OpMode.TExd11,
#         help=OpMode.__doc__,
#         prompt=f"""2. Choose operating mode""",
#     ),
#     bw: float = typer.Option(
#         0.05,
#         help="""Desired minimum impedance bandwidth""",
#         prompt="""3. Enter the minmum factional bandwith (e.g.: 0.05 for 5%)""",
#     ),
#     vswr: float = typer.Option(
#         2.0,
#         help="""Tolerable voltage standing wave ratio (VSWR)""",
#         prompt="""4. Enter the VSWR for the impedance bandwidth calculations (e.g.: 2)""",
#     ),
#     er: float = typer.Option(
#         10.4,
#         help="Material dielectric constant",
#         prompt="""5. Enter DR's dielectric constant""",
#     ),
#     wh: str = typer.Option(
#         "1,0.1,2",
#         help="""Width/height ratio of the resonator (facultative)""",
#         prompt="""Enter the **width**/**height** (w/h) ratio of the DR (e.g.: 1,0.1,2)""",
#     ),
#     dh: str = typer.Option(
#         "1,0.1,2",
#         help="""Depth/height ratio of the resonator (facultative)""",
#         prompt="""Enter the **depth**/**height** (d/h) ratio of the DR (e.g.: 1,0.1,2)""",
#     ),
# ):
#     """**Design** a rectangular resonator."""

#     # Convert to numpy data
#     f = np.double(freq)  # """Frequency (GHz)"""
#     bandwidth = np.double(bw)
#     v = np.double(vswr)
#     epsilon_r = np.double(er)
#     wh_list = np.array(wh.split(","), dtype=np.double)
#     dh_list = np.array(dh.split(","), dtype=np.double)
#     w_h = d_h = np.double(0)
#     w_h_step = d_h_step = np.double(0)
#     w_h_min = w_h_max = d_h_min = d_h_max = np.double(0)
#     if len(wh_list) > 3:
#         raise typer.Exit(1)
#     elif len(wh_list) == 3:
#         [w_h_min, w_h_step, w_h_max] = wh_list
#     elif len(wh_list) == 1:
#         [w_h] = wh_list
#     if len(dh_list) > 3:
#         raise typer.Exit(1)
#     elif len(dh_list) == 3:
#         [d_h_min, d_h_step, d_h_max] = dh_list
#     elif len(dh_list) == 1:
#         [d_h] = dh_list
#     # Image effect
#     w_h_min = w_h_min / 2
#     w_h_max = w_h_max / 2
#     d_h_min = w_h_min / 2
#     d_h_max = d_h_max / 2

#     # Maximum Q factor for the minimum bandwidth and VSWR
#     q_max = (v - 1) / (np.sqrt(v) * bandwidth)

#     k0 = 2 * np.pi * f * 1e7 / c

#     result_data = np.array([])

#     if len(wh_list) == len(dh_list) == 3:
#         for k in np.arange(w_h_min, w_h_max, w_h_step):
#             for l in np.arange(d_h_min, d_h_max, d_h_step):
#                 if mode == OpMode.TExd11:
#                     w, d, h, q = solve_TExd11(f, epsilon_r, k, l, k0)
#                     if q > q_max:
#                         # print(
#                         #     """The desired bandwidth cannot be achieved in this mode
#                         #     with the specified dielectric constant and dimensions."""
#                         # )
#                         # raise typer.Exit(1)
#                         continue
#                     bw_actual = (v - 1) / (np.sqrt(v) * q) * 100
#                     result_data = np.append(
#                         result_data, [k**2, l**2, w, d, h, q, bw_actual]
#                     )

#     result = Table(title="Design Result")
#     result.add_column("width/height")
#     result.add_column("depth/height")
#     result.add_column("width (mm)")
#     result.add_column("depth (mm)")
#     result.add_column("height (mm)")
#     result.add_column("Q factor", justify="right", style="cyan")
#     result.add_column("bandwidth", justify="right", style="magenta")

#     for result_row in result_data:
#         result.add_row(result_row)

#     print(result)


# @app.command(name="analyze")
# def cli_analyze():
#     """**Analyze** a rectangular resonator."""



# from typing import Optional
# from types import ModuleType

# import typer

# from . import cyl, hemi, rect, ring

# from ... import __version__

# epilog = """
#    Made with ❤ . Cli app supported by `Typer`.
#    """


# dra_types: dict[str, tuple[ModuleType, str]] = {
#    "rect": (rect, "rectangular"),
#    "cyl": (cyl, "cylindrical"),
#    "hemi": (hemi, "hemispherical"),
#    "ring": (ring, "ring"),
# }


# def add_command(dra_type: str):
#    (module, name) = dra_types[dra_type]
#    app.add_typer(
#        module.app,
#        help=f"""**Deign**/**analyze** a **{name}** resonator.""",
#        no_args_is_help=True,
#        invoke_without_command=True,
#        epilog=epilog,
#    )


# def version_callback(value: bool):
#    if value:
#        print(f"version: {__version__}")
#        raise typer.Exit()


# app = typer.Typer(epilog=epilog, rich_markup_mode="markdown", no_args_is_help=True)

# list(map(add_command, dra_types))


# @app.callback()
# def main(
#    version: Optional[bool] = typer.Option(
#        None,
#        "--version",
#        "-v",
#        help="Show version.",
#        callback=version_callback,
#        is_eager=True,
#    ),
# ):
#    """
#    🧮 Dielectric Resonator (DR) Calculator

#    Run **cli COMMAND --help** to show command usage.
#    """
#    ...


# if __name__ == "__main__":
#    app()



"""Design tools.
"""

from pyaedt.modules.MaterialLib import Materials


def import_materials(materials: Materials, material: str | list[str]) -> None:
    """Import materials from the system lib.

    :param pyaedt.modules.MaterialLib.Materials materials: AEDT material database
    :param str | list[str] material: The name of materials to be imported
    """
    if not isinstance(material, list):
        material = [material]
    for mat in material:
        materials.checkifmaterialexists(mat)



"""Generating report."""

# %% Import
import numpy as np
from pyaedt.modules.AdvancedPostProcessing import PostProcessor
from typing import cast

from pyaedt.modules.solutions import SolutionData
from pyaedt.generic.constants import SOLUTIONS


# %% Functions
def get_s_params(
    post: PostProcessor, row: int, col: int, setup_name: str, sweep_name: str
) -> np.ndarray:
    """Fetch S parameters as an array.

    :param pyaedt.modules.AdvancedPostProcessing.PostProcessor post: Advanced post processor
    :param int row: Which row of the S matrix
    :param int col: Which column of the S matrix
    :raises AssertionError: Check if the result id real
    :return np.ndarray: S parameters in dB
    """

    match post.post_solution_type:
        case SOLUTIONS.Hfss.DrivenModal:
            category = "Modal Solution Data"
        case SOLUTIONS.Hfss.DrivenTerminal:
            category = "Terminal Solution Data"
        case _:
            category = "Modal Solution Data"

    s = cast(
        SolutionData,
        post.get_solution_data(
            f"dB(S({row},{col}))",
            f"{setup_name} : {sweep_name}",
            "Sweep",
            report_category=category,
        ),
    )
    assert s.is_real_only(), "S parameters is not real only."
    return np.array(s.data_real())



# import typer

# app = typer.Typer()


# @app.callback()
# def ring():
#     ...


# @app.command()
# def design():
#     """**Design** a ring resonator."""


# @app.command()
# def analyze():
#     """**Analyze** a ring resonator."""



# import typer

# app = typer.Typer()

# @app.callback()
# def hemi():
#     ...

# @app.command()
# def design():
#     """**Design** a hemispherical resonator."""


# @app.command()
# def analyze():
#     """**Analyze** a hemispherical resonator."""



"""
Copy Design Variables

Description:
    Copy local variables in the current design into clipboard.
    Only works on Windows.
"""

import os
import ScriptEnv


oProject = oDesktop.GetActiveProject()
oDesign = oProject.GetActiveDesign()
oDesignVariables = oDesign.GetChildObject("Variables")
variables = oDesign.GetVariables()

project_name = oProject.GetName()
design_name = oDesign.GetName()


def log(msg):
    oDesktop.AddMessage(project_name, design_name, 0, msg, "User Toolkit")


variables_dict = {}

for var in variables:
    variables_dict.update({var: oDesign.GetVariableValue(var)})

log("Exporting design variables:")
log(str(variables_dict))

os.system("echo " + str(variables_dict).strip() + " | clip")

log("Design variables has been copied to clipboard.")



import os
import sys

from pyaedt import settings
from pyaedt.generic.desktop_sessions import _desktop_sessions


class Desktop:
    """Provides the Ansys Electronics Desktop (AEDT) interface."""

    def __new__(cls):
        if len(_desktop_sessions.keys()) > 0:
            # Reuse _desktop_session[sessions[0]]
            ...
        else:
            return object.__new__(cls)

    def __init__(
        self,
        specified_version: str | None = None,
        non_graphical: bool = False,
        new_desktop_session: bool = True,
        close_on_exit: bool = True,
        student_version: bool = False,
        machine: str = "",
        port: int = 0,
        aedt_process_id: int | None = None,
    ):
        self._main = sys.modules["__main__"]

        student_version_flag, version_key, version = self._assert_version(
            specified_version, student_version
        )

        # AEDT opening decision tree
        if "oDesktop" in dir():
            starting_mode = "console_in"
        elif "oDesktop" in dir(self._main) and self._main.oDesktop is not None:
            starting_mode = "console_out"
        else:
            ...

        starting_mode = "grpc"

        # starting AEDT
        settings.aedt_version = version_key
        if "oDesktop" in dir(self._main):
            del self._main.oDesktop
        if starting_mode == "grpc":
            self._init_grpc(
                non_graphical,
                new_desktop_session,
                version,
                student_version_flag,
                version_key,
            )

        settings.enable_desktop_logs = not non_graphical
        self._init_desktop()

    def _assert_version(
        self,
        specified_version: str | None = None,
        student_version: bool = False,
    ):
        return False, "2023.2", "Ansoft.ElectronicsDesktop.2023.2"

    def _init_grpc(
        self,
        non_graphical: bool,
        new_desktop_session: bool,
        version: str,
        student_version: bool,
        version_key: str,
    ):
        self.machine = ""

        self.port = _find_free_port()

        installer = os.path.join(
            self._main.sDesktopinstallDirectory, "ansysedt.exe"
        )

        out, self.port = launch_aedt(
            installer, non_graphical, self.port, student_version
        )
        self.launched_by_pyaedt = True
        oApp = self._initialize(
            is_grpc=True,
            non_graphical=non_graphical,
            machine=self.machine,
            port=self.port,
            new_session=not out,
            version=version_key,
        )

        self._main.isoutsideDesktop = True
        self._main.oDekstop = oApp.GetAppDesktop()
        _proc = self._main.oDesktop.GetProcessID()
        self.is_grpc_api = True

    def _initialize(
        self,
        machine: str = "",
        port: int = 0,
        non_graphical: bool = False,
        new_session: bool = False,
        version: str | None = None,
        is_grpc: bool = True,
    ):
        base_path = self._main.sDesktopinstallDirectory
        sys.path.insert(0, base_path)
        sys.path.insert(
            0, os.path.join(base_path, "PythonFiles", "DesktopPlugin")
        )
        ...
        import pyaedt.generic.grpc_plugin as StandalonePyScriptWrapper

        return StandalonePyScriptWrapper.CreateAedtApplication(
            machine, port, non_graphical, new_session
        )

    def _init_desktop(self):
        self._main.pyaedt_version = pyaedtversion
        self._main.AEDTVersion = self._main.oDesktop.GetVersion()[0:6]
        self._main.oDesktop.RestoreWindow()
        self._main.sDesktopinstallDirectory = self._main.oDesktop.GetExeDir()
        self._main.pyaedt_initialized = True

def _find_free_port():
    from contextlib import closing
    import socket

    with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as s:
        s.bind(("127.0.0.1", 0))
        s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        return s.getsockname()[1]


def launch_aedt(
    full_path: str,
    non_graphical: bool,
    port: int,
    student_version: bool,
    first_run: bool = True,
):
    """Launch AEDT in gRPC mode."""

    return True, port
